/*
  Copyright 2019 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.java.sample;

import androidx.appcompat.app.AppCompatActivity;

import android.os.Bundle;
import android.util.Log;

import com.kakao.sdk.auth.LoginClient;
import com.kakao.sdk.auth.LoginClientKt;
import com.kakao.sdk.auth.model.OAuthToken;
import com.kakao.sdk.partner.talk.TalkApiClientKt;
import com.kakao.sdk.partner.talk.model.Chat;
import com.kakao.sdk.partner.talk.model.Chats;
import com.kakao.sdk.talk.TalkApiClient;
import com.kakao.sdk.user.UserApiClient;
import com.kakao.sdk.user.UserApiClientKt;
import com.kakao.sdk.user.model.User;

import io.reactivex.SingleSource;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.functions.Consumer;
import io.reactivex.functions.Function;
import kotlin.Unit;
import kotlin.jvm.functions.Function2;

public class MainActivity extends AppCompatActivity {

    private final String TAG = "KakaoSdkInJava";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // 카카오톡으로 로그인
        LoginClient.getInstance().loginWithKakaoTalk(this, new Function2<OAuthToken, Throwable, Unit>() {
            @Override
            public Unit invoke(OAuthToken token, Throwable error) {
                if (error != null) {
                    Log.e(TAG, "로그인 실패", error);
                } else {
                    Log.d(TAG, "로그인 성공");

                    // 사용자 정보 요청
                    UserApiClient.getInstance().me(new Function2<User, Throwable, Unit>() {
                        @Override
                        public Unit invoke(User user, Throwable error) {
                            if (error != error) {
                                Log.e(TAG, "사용자 정보 요청 실패", error);
                            } else {
                                Log.i(TAG, user.toString());
                            }
                            return null;
                        }
                    });
                }
                return null;
            }
        });
        LoginClientKt.getRx(LoginClient.Companion).loginWithKakaoAccount(this)
                .flatMap(new Function<OAuthToken, SingleSource<User>>() {
                    @Override
                    public SingleSource<User> apply(OAuthToken oAuthToken) throws Exception {
                        return UserApiClientKt.getRx(UserApiClient.Companion).me();
                    }
                })
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(new Consumer<User>() {
                    @Override
                    public void accept(User user) throws Exception {
                        Log.i(TAG, user.toString());
                    }
                }, new Consumer<Throwable>() {
                    @Override
                    public void accept(Throwable throwable) throws Exception {
                        Log.e(TAG, throwable.toString());
                    }
                });
        TalkApiClientKt.chatList(TalkApiClient.getInstance(), new Function2<Chats, Throwable, Unit>() {
            @Override
            public Unit invoke(Chats chats, Throwable throwable) {
                if (throwable != null) {
                    Log.e(TAG, throwable.toString());
                } else {
                    for (Chat element : chats.getElements()) {
                        Log.d(TAG, element.toString());
                    }
                }
                return null;
            }
        });
    }
}
